@@ -30,9 +30,6 @@ import time
 import logging
 import weakref
 
-builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
-pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
-
 import dbus
 import _dbus_bindings
 import dbus.glib
@@ -48,13 +45,18 @@ except ImportError:
 
 logging.basicConfig()
 
+if 'DBUS_TEST_UNINSTALLED' in os.environ:
+    builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
+    pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
+    pkg = dbus.__file__
 
-pkg = dbus.__file__
-if not pkg.startswith(pydir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%pkg)
+    if not pkg.startswith(pydir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % pkg)
 
-if not _dbus_bindings.__file__.startswith(builddir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%_dbus_bindings.__file__)
+    if not _dbus_bindings.__file__.startswith(builddir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % _dbus_bindings.__file__)
 
 test_types_vals = [1, 12323231, 3.14159265, 99999999.99,
                  "dude", "123", "What is all the fuss about?", "gob@gob.com",
