@@ -28,13 +28,11 @@ run in isolation.
 
 from __future__ import unicode_literals
 
+import struct
 import sys
 import os
 import unittest
 
-builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
-pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
-
 import _dbus_bindings
 import dbus
 import dbus.lowlevel as lowlevel
@@ -48,21 +46,59 @@ else:
     def make_long(n):
         return long(n)
 
+if 'DBUS_TEST_UNINSTALLED' in os.environ:
+    builddir = os.path.normpath(os.environ["DBUS_TOP_BUILDDIR"])
+    pydir = os.path.normpath(os.environ["DBUS_TOP_SRCDIR"])
+    pkg = dbus.__file__
+
+    if not pkg.startswith(pydir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % pkg)
+
+    if not _dbus_bindings.__file__.startswith(builddir):
+        raise Exception("DBus modules (%s) are not being picked up from the "
+                "package" % _dbus_bindings.__file__)
+
+    assert _dbus_bindings.__version__ == os.environ['DBUS_PYTHON_VERSION'], \
+            '_dbus_bindings was compiled as version %s but Automake says '\
+            'we should be version %s' \
+            % (_dbus_bindings.__version__, os.environ['DBUS_PYTHON_VERSION'])
 
-# Check that we're using the right versions
-if not dbus.__file__.startswith(pydir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%dbus.__file__)
-if not _dbus_bindings.__file__.startswith(builddir):
-    raise Exception("DBus modules (%s) are not being picked up from the package"%_dbus_bindings.__file__)
 assert (_dbus_bindings._python_version & 0xffff0000
         == sys.hexversion & 0xffff0000), \
         '_dbus_bindings was compiled for Python %x but this is Python %x, '\
         'a different major version'\
         % (_dbus_bindings._python_version, sys.hexversion)
-assert _dbus_bindings.__version__ == os.environ['DBUS_PYTHON_VERSION'], \
-        '_dbus_bindings was compiled as version %s but Automake says '\
-        'we should be version %s' \
-        % (_dbus_bindings.__version__, os.environ['DBUS_PYTHON_VERSION'])
+
+def uni(x):
+    """Return a Unicode string consisting of the single Unicode character
+    with the given codepoint (represented as a surrogate pair if this is
+    a "narrow" Python build). This is a generalization of unichr().
+
+    uni(0x0001f639) == u'\\U0001f639' in versions where that syntax is
+    supported.
+    """
+    if x <= 0xFFFF:
+        if is_py3:
+            return chr(x)
+        else:
+            return unichr(x)
+    elif is_py3:
+        return struct.pack('>I', x).decode('utf_32_be')
+    else:
+        # Python 2.6 didn't accept unicode format strings
+        return struct.pack(b'>I', x).decode('utf_32_be')
+
+def utf8(*xs):
+    """Return a bytestring containing the given UTF-8 bytes.
+
+    utf8(0xc2, 0xa3) == b'\\xc2\\xa3' on Python versions that
+    allow bytestring literals (but some Python 2 versions do not).
+    """
+    if is_py3:
+        return bytes(xs)
+    else:
+        return str('').join(map(chr, xs))
 
 class TestTypes(unittest.TestCase):
 
@@ -425,19 +461,32 @@ class TestMessageMarshalling(unittest.TestCase):
 
     def test_utf8(self):
         from _dbus_bindings import SignalMessage
-        if is_py3:
-            def utf8(*xs):
-                return bytes(xs)
-            def uni(x):
-                return chr(x)
-        else:
-            def utf8(*xs):
-                return str('').join(map(chr, xs))
-            def uni(x):
-                return unichr(x)
+
         for bad in [
                 uni(0xD800),
                 utf8(0xed, 0xa0, 0x80),
+                ]:
+            s = SignalMessage('/', 'foo.bar', 'baz')
+            try:
+                s.append(bad, signature='s')
+            except UnicodeError:
+                pass
+            else:
+                raise AssertionError('Appending %r should fail' % bad)
+        for good in [
+                uni(0xfdcf),
+                uni(0xfdf0),
+                uni(0xfeff),
+                uni(0x0001feff),
+                uni(0x00020000),
+                uni(0x0007feff),
+                uni(0x00080000),
+                uni(0x0010feff),
+                ]:
+            s = SignalMessage('/', 'foo.bar', 'baz')
+            s.append(good, signature='s')
+            s.append(good.encode('utf-8'), signature='s')
+        for noncharacter in [
                 uni(0xFDD0),
                 utf8(0xef, 0xb7, 0x90),
                 uni(0xFDD7),
@@ -463,24 +512,11 @@ class TestMessageMarshalling(unittest.TestCase):
                 ]:
             s = SignalMessage('/', 'foo.bar', 'baz')
             try:
-                s.append(bad, signature='s')
+                s.append(noncharacter, signature='s')
             except UnicodeError:
-                pass
+                pass  # libdbus < 1.6.10 disallows noncharacters
             else:
-                raise AssertionError('Appending %r should fail' % bad)
-        for good in [
-                uni(0xfdcf),
-                uni(0xfdf0),
-                uni(0xfeff),
-                uni(0x0001feff),
-                uni(0x00020000),
-                uni(0x0007feff),
-                uni(0x00080000),
-                uni(0x0010feff),
-                ]:
-            s = SignalMessage('/', 'foo.bar', 'baz')
-            s.append(good, signature='s')
-            s.append(good.encode('utf-8'), signature='s')
+                pass  # libdbus >= 1.6.10 allows noncharacters
 
 class TestMatching(unittest.TestCase):
     def setUp(self):
@@ -500,6 +536,20 @@ class TestMatching(unittest.TestCase):
         self._message.append('/', signature='o')
         self.assertFalse(self._match.maybe_handle_message(self._message))
 
+class TestVersion(unittest.TestCase):
+    if sys.version_info[:2] < (2, 7):
+        def assertGreater(self, first, second):
+            self.assertTrue(first > second)
+
+        def assertLess(self, first, second):
+            self.assertTrue(first < second)
+
+    def test_version(self):
+        self.assertGreater(dbus.version, (0, 41))
+        self.assertLess(dbus.version, (23, 0))
+        self.assertGreater(dbus.__version__, '0')
+        self.assertLess(dbus.__version__, '9')
+
 if __name__ == '__main__':
     # Python 2.6 doesn't accept a `verbosity` keyword.
     kwargs = {}
